@@ -379,6 +379,12 @@ class CAE_B3U4I:
         self.phase[2] = (bB_update.updated[self.node][self.offset_miso + 15] << 8) + bB_update.updated[self.node][
             self.offset_miso + 16]
         self.phase = [i / 20480 * 360 for i in self.phase]
+        # Check if Phase is over 180 degree.
+        x = range(3)
+        for i in x:
+            if self.phase[i] > 180:
+                self.phase[i] -= 360
+
         return self.phase
 
     def readCurrent(self):
