@@ -3,25 +3,29 @@
 import threading
 import time
 import math
+import json
 from datetime import datetime
 # Import pymodbus for the Modbus RTU Module
 from pymodbus.client.sync import ModbusSerialClient as ModbusClient
 lock = threading.Lock()
-filename = 'error.txt'
+
+filename = "error.txt"
+error = open("error.txt", "w+")
 
 
 def splitBytes(list16bit):
-    # Function to split a list with 16 bit elements into 8 bit element list
+    ''' Function to split a list with 16 bit elements into 8 bit element list '''
     list8bit = []
     for value in list16bit:
-        a = (value >> 8) & 0xff
-        b = value & 0xff
+        a = (value >> 8) & 0xFF
+        b = value & 0xFF
         list8bit.append(a)
         list8bit.append(b)
     return list8bit
 
 
 def addBytes(list8bit):
+    ''' Function to add two 8 bit element from a list to one 16 bit element and put them in a list '''
     list16bit = []
     if not len(list8bit) // 2:
         list8bit.append(0)
@@ -36,7 +40,7 @@ def addBytes(list8bit):
 
 
 class local_master:
-    # Sorting the information about the local master
+    ''' Sorting the information about the local master '''
     def __init__(self):
         self.info = {}
 
@@ -51,10 +55,12 @@ class local_master:
                          'Reserved1': bB_update.local_data[id][7],
                          'Reserved2': bB_update.local_data[id][8]}
 
+
 local = local_master()
 
+
 class slave_module:
-    # Sorting the Information about the emBricks
+    ''' Sorting the Information about the Bricks '''
     def __init__(self):
         self.status = {}
         self.data_length_mosi = {}
@@ -77,6 +83,8 @@ class slave_module:
                 self.manu_id[id, i] = (bB_update.local_data[id][9+(i*11)+7] << 8) + bB_update.local_data[id][9+(i*11)+8]
                 self.offset_mosi[id, i] = bB_update.local_data[id][9+(i*11)+9]
                 self.offset_miso[id, i] = bB_update.local_data[id][9+(i*11)+10]
+
+
 slave = slave_module()
 
 class connection:
@@ -97,31 +105,26 @@ class connection:
         print("emBRICK(R), Starterkit Modbus RTU Python\n")
         print("(c) 2021 by IMACS GmbH \n")
         print("---------------------------------------------------\n")
-        error = open(filename, 'wt')
-
         # Connecting to the LWC's on the given Slave Adress
         self.node = ModbusClient(method='rtu', port=self.port, timeout= self.timeout, baudrate= self.baudrate)
         error.write(f'Error Log\nCommunication over Modbus RTU with Python\
          Baudrate: {self.baudrate}, \tPort: {self.port}, \tTimeout: {self.timeout}\n\n')
-
         if self.node.is_socket_open() == True:
             self.node.close()
         connected = self.node.connect()
         if not connected:
-            error.write(f'Connection with Port {self.port} failed!')
+            error.write(f'{datetime.now()}\tERROR: Connection with Port {self.port} failed!')
             error.close()
             exit()
-        error.close
         # Initialize the Functions to start the Program
         bB_update.getLWCsInfo()
         local.data()
         slave.info()
         bB_update.update_first()
         bB.createEmptyList()
-        t1 = threading.Thread(target=bB_update.update)
-
-        t1.start()
-        init()
+        updatcycle = threading.Thread(target=bB_update.update)
+        updatcycle.start()
+        #init()
         #time.sleep(1)
         #t1.join()
 
@@ -134,7 +137,7 @@ class brickBus_communication:
         self.local_data = {}
         self.updated = {}
         self.updates = {}
-
+        self.stopThread = False
     def getLWCsInfo(self):
 
         for id in connect.unit_id:
@@ -144,24 +147,29 @@ class brickBus_communication:
                 data_16byte = responce.registers
                 self.local_data[id] = splitBytes(data_16byte)
             else:
-                error = open(filename, 'at')
-                error.write(f'{datetime.now()}\tConnection to the Coupling-Master with Modbus Address {id} failed!\n')
-                error.close()
+                error.write(f'{datetime.now()}\tERROR: Connection to the Coupling-Master with Modbus Address {id} failed!\n')
+                connect.unit_id.remove(id)
 
     def update_first(self, ):
-        # Update the emBricks in the individual updaterate
+        ''' Update the emBricks in the individual updaterate '''
         for id in connect.unit_id:
-            responce = connect.node.read_input_registers(0, 120, unit=id)
-            time.sleep(0.005)
-            if not responce.isError():
-                self.updates[id] = responce.registers
-            else:
-                error = open(filename, 'at')
-                error.write(f'{datetime.now()}\tChecksum Error\n')
-                error.close()
-            if self.updates[id] != 0:
-                self.updated[id] = splitBytes(self.updates[id])
-        #bB.createEmptyList()
+            i = 0
+            while(i < 5):
+                responce = connect.node.read_input_registers(0, 120, unit=id)
+                time.sleep(0.005)
+                if not responce.isError():
+                    self.updates[id] = responce.registers
+                else:
+                    error.write(f'{datetime.now()}\tERROR: Checksum Error or Timeout\n')
+                    i += 1
+                if self.updates[id] != 0:
+                    self.updated[id] = splitBytes(self.updates[id])
+                    break
+            if i == 5:
+                error.write(f'{datetime.now()}\tERROR: Connection with LWCs on Modbus Address {id} not possible\n')
+                connect.unit_id.remove(id)
+        error.close()
+
 
     def update(self):
         # Update the emBricks in the individual updaterate
@@ -184,20 +192,22 @@ class brickBus_communication:
                     self.updates[id] = responce.registers
                 else:
                     error = open(filename, 'at')
-                    error.write(f'{datetime.now()}\tChecksum Error\n')
+                    error.write(f'{datetime.now()}\tERROR: Checksum Error or Timeout Error\n')
                     error.close()
 
                 if self.updates[id] != 0:
                     lock.acquire()
                     self.updated[id] = splitBytes(self.updates[id])
                     lock.release()
+                if self.stopThread:
+                    raise SystemExit()
 
 bB_update = brickBus_communication()
 
 
 
 class functions:
-    # Implement the 6 Main Function to Control the emBricks
+    ''' Implement the 6 Main Function to Control the Bricks '''
     def __init__(self, ):
         self.gS = 0
         self.gByte = 0
@@ -208,6 +218,9 @@ class functions:
         self.put = {}
         self.set = {}
         self.buffer_length = {}
+        self.printed_gBit = False
+        self.printed_gB = False
+        self.printed_gS = False
         self.printed_pBit = False
         self.printed_pB = False
         self.printed_pS = False
@@ -224,7 +237,7 @@ class functions:
             self.buffer_length[id] = input
 
     def getShort(self, node, module, bytePos):
-        # Return the value in Size of 2 bytes(16bit) of the desired Byte Position
+        ''' Return the value in Size of 2 bytes(16bit) of the desired Byte Position '''
         if  ((node,module-1) in slave.offset_miso):
             lock.acquire()
             byte1 = (bB_update.updated[node][slave.offset_miso[node, module-1] + bytePos + 1]) << 8
@@ -233,10 +246,16 @@ class functions:
             self.gS = byte1 + byte2
         else:
             self.gS = None
+            if self.printed_gS == False:
+                error.open(filename, "a+")
+                error.write(
+                    f'{datetime.now()}\tERROR: getShort(), ID: {node} Brick: {module} BytePos: {bytePos}\n')
+                error.close()
+                self.printed_gS = True
         return self.gS
 
     def getByte(self, node, module, bytePos):
-        # Return the value in Size of 1 byte(8bit) of the desired Byte Position
+        ''' Return the value in Size of 1 byte(8bit) of the desired Byte Position '''
         if  ((node,module-1) in slave.offset_miso):
             #print(bB_update.updated)
             byte = slave.offset_miso[node, module-1] + bytePos + 1
@@ -245,10 +264,15 @@ class functions:
             lock.release()
         else:
             self.gByte = None
+            if self.printed_gB == False:
+                error.open(filename, "a+")
+                error.write(f'{datetime.now()}\tERROR: getByte(), ID: {node} Brick: {module} BytePos: {bytePos}\n')
+                error.close()
+                self.printed_gB = True
         return self.gByte
 
     def getBit(self, node, module, bytePos, bitPos):
-        # Return the value in Size of a Bit of the desired Byte -> Bit Position
+        ''' Return the value in Size of a Bit of the desired Byte -> Bit Position '''
         if  ((node,module-1) in slave.offset_miso):
             lock.acquire()
             byte = bB_update.updated[node][slave.offset_miso[node, module-1] + bytePos + 1]
@@ -256,11 +280,17 @@ class functions:
             self.gBit = 1 if (byte & (1 << bitPos)) else 0
         else:
             self.gBit = None
+            if self.printed_gBit == False:
+                error.open(filename, "a+")
+                error.write(
+                    f'{datetime.now()}\tERROR: getBit(), ID {node} Brick: {module} BytePos: {bytePos} BitPos: {bitPos}\n')
+                error.close()
+                self.printed_gBit = True
         return self.gBit
 
 
     def putShort(self, node, module, bytePos, value):
-        # Put the desired value in Size 2 bytes to the desired Byte Position
+        ''' Put the desired value in Size 2 bytes to the desired Byte Position '''
         try:
             output = slave.offset_mosi[node, module - 1] + bytePos
             output1 = (value >> 8) & 0xFF
@@ -271,14 +301,13 @@ class functions:
         except:
             if self.printed_pS == False:
                 error = open(filename, 'at')
-                error.write(f'Node: {node}, Module: {module}, BytePos: {bytePos} not found!\n')
+                error.write(f'{datetime.now()}\tERROR: putShort(), ID: {node} Brick: {module} BytePos: {bytePos}\n')
                 error.close()
                 self.printed_pS = True
 
 
     def putByte(self, node, module, bytePos, value):
-        # Put the desired value in Size 1 bytes to the desired Byte Position
-
+        ''' Put the desired value in Size 1 bytes to the desired Byte Position '''
         try:
             output = slave.offset_mosi[node, module - 1] + bytePos
             self.pByte = value
@@ -286,12 +315,12 @@ class functions:
         except:
             if self.printed_pB == False:
                 error = open(filename, 'at')
-                error.write(f'Node: {node}, Module: {module}, BytePos: {bytePos} not found!\n')
+                error.write(f'{datetime.now()}\tERROR: putByte(), ID: {node} Module: {module} BytePos: {bytePos}\n')
                 error.close()
                 self.printed_pB = True
 
     def putBit(self, node, module, bytePos, bitPos, value):
-        # Put the desired value in Size 1 bit to the desired Byte -> Bit Position
+        ''' Put the desired value in Size 1 bit to the desired Byte -> Bit Position '''
         try:
             output = slave.offset_mosi[node, module - 1] + bytePos
             self.pBit = 1 << bitPos
@@ -315,12 +344,57 @@ bB = functions()
 
 
 def init():
-    #Prints the Information about the Local Master and the connected Bricks to him
+    ''' Prints the Information about the Local Master and the connected Bricks '''
+    #Build the struct of the json data
+    info = {
+        # 1: Is the Modbus Adress
+        1: {
+            # In Master: Information from LWCs
+            'Master': {
+                'number of connected bricks': 0,
+                'status of the local-master': 0,
+                'component ID of the local-master': 0,
+                'remote bus protocol version': 0,
+                'software version of local-master': 0,
+                'Local Master manufacturer ID': 0,
+                'Reserved1': 0,
+                'Reserved2': 0
+            },
+            # In Bricks: Information from connected Bricks on LWCs
+            'Bricks':
+                {
+                    1: {
+                        'status of slave-module': 0,
+                        'Data Length MOSI': 0,
+                        'Data length MISO': 0,
+                        'Slave local bus protocol version': 0,
+                        'Slave hardware revision': 0,
+                        'Slave device ID': 0,
+                        'Slave manufacturer ID': 0,
+                        'Data offset MOSI': 0,
+                        'Data offset MISO': 0
+                    }
+                }
+        }
+    }
+    #Prints the Information about the Local Master and the connected Bricks
     for id in connect.unit_id:
+        info[id]['Master'] = {}
+        info[id]['Master'] = local.info[id]
         print(f'\nNode {id} is connected with the local master: '
               f'{local.info[id]["component ID of the local-master"] // 1000}-{local.info[id]["component ID of the local-master"] % 1000} '
               f'with Software Version: {local.info[id]["software version of local-master"]}')
-        print(f'With following modules:')
         for i in range(local.info[id]['number of connected bricks']):
-            print(f'{i + 1}: {slave.id[id, i] // 1000}-{slave.id[id, i] % 1000} with Hardware Revision: {slave.hard_rev[id, i]}')
-    print("To abort the program please press STRG + C\n\n")
+            info[id]['Bricks'][i] = {}
+            info[id]['Bricks'][i]['status of slave-module'] = slave.status[id, i]
+            info[id]['Bricks'][i]['Data Length MOSI'] = slave.data_length_mosi[id, i]
+            info[id]['Bricks'][i]['Data Length MISO'] = slave.data_length_miso[id, i]
+            info[id]['Bricks'][i]['Slave local bus protocol version'] = slave.pro_ver[id, i]
+            info[id]['Bricks'][i]['Slave hardware revision'] = slave.hard_rev[id, i]
+            info[id]['Bricks'][i]['Slave device ID'] = slave.id[id, i]
+            info[id]['Bricks'][i]['Slave manufacturer ID'] = slave.manu_id[id, i]
+            info[id]['Bricks'][i]['Data offset MOSI'] = slave.offset_mosi[id, i]
+            info[id]['Bricks'][i]['Data offset MISO'] = slave.offset_miso[id, i]
+    #json_object = json.dumps(info,indent=4)
+    with open("info.json", "w") as outfile:
+        json.dump(info, outfile)
